package kutils

import (
	"crypto/tls"
	"crypto/x509"
	"encoding/json"
	"github.com/Shopify/sarama"
	"io/ioutil"
	"strings"
)

type ProducerContext struct {
	KafkaCtx
}

//Функции записи

// SetWriter
//С его помощью можно записывать уже готовое соединение минуя SetConnectionWriter
func (ctx *KafkaCtx) SetWriter(w sarama.SyncProducer) *ProducerContext {
	prodCtx := ProducerContext{*ctx}
	if prodCtx.Error != nil {
		return &prodCtx
	}
	prodCtx.w = w
	return &prodCtx
}

func (ctx *KafkaCtx) WriterConf(user, password, certificatePath string) *ProducerContext {
	prodCtx := ProducerContext{*ctx}
	if prodCtx.Error != nil {
		return &prodCtx
	}
	conf := sarama.NewConfig()
	conf.Producer.RequiredAcks = sarama.WaitForAll
	conf.Producer.Return.Successes = true
	conf.Version = sarama.MaxVersion
	conf.ClientID = "sasl_scram_client"
	conf.Net.SASL.Enable = true
	conf.Net.SASL.Handshake = true
	conf.Net.SASL.User = user
	conf.Net.SASL.Password = password
	conf.Net.SASL.SCRAMClientGeneratorFunc = func() sarama.SCRAMClient { return &XDGSCRAMClient{HashGeneratorFcn: SHA512} }
	conf.Net.SASL.Mechanism = sarama.SASLMechanism(sarama.SASLTypeSCRAMSHA512)

	certs := x509.NewCertPool()
	pemData, err := ioutil.ReadFile(certificatePath)
	if err != nil {
		prodCtx.Error = err
		return &prodCtx
	}
	certs.AppendCertsFromPEM(pemData)
	conf.Net.TLS.Enable = true
	conf.Net.TLS.Config = &tls.Config{
		InsecureSkipVerify: true,
		RootCAs: certs,
	}

	prodCtx.sslConf = conf

	syncProd, err := sarama.NewSyncProducer(strings.Split(prodCtx.connection.host, ","), prodCtx.sslConf)
	if err != nil {
		prodCtx.Error = err
		return &prodCtx
	}
	return prodCtx.KafkaCtx.SetWriter(syncProd)
}

func (ctx *ProducerContext) SetKey(key string) *ProducerContext {
	if ctx.Error != nil {
		return ctx
	}
	ctx.key = key
	return ctx
}

func (ctx *ProducerContext) SetValue(object KutilMessage) *ProducerContext {
	if ctx.Error != nil {
		return ctx
	}
	ctx.value = object
	return ctx
}

func (ctx *ProducerContext) SendMessage() *ProducerContext {
	if ctx.Error != nil {
		return ctx
	}
	message, err := json.Marshal(ctx.value)
	if err != nil {
		ctx.Error = err
		return ctx
	}
	_, _, err = ctx.w.SendMessage(
		&sarama.ProducerMessage{Topic: ctx.connection.topic,
			Key: sarama.StringEncoder(ctx.key),
			Value: sarama.StringEncoder(message)},
	)
	return ctx
}

func (ctx *ProducerContext) Close() *ProducerContext {
	if ctx.Error != nil {
		return ctx
	}
	ctx.Error = ctx.w.Close()
	return ctx
}